#!/usr/bin/env ruby

MOD_VERSION = "ADVNACE MODULE version 0.1"

#############################################################
# author:  mathias gumz, curt micol
# date  :  04/08/06 13:57:10
# file  :  mod_advanced.rb
# about :  advanced module for rweather

#############################################################
# Copyright (c) 2004 Mathias Gumz <gumz at cs.uni-magdeburg.de>
# Copyright (c) 2004 Curt Micol <asenchi at asenchi.com>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#############################################################

OPTIONS_TEXT = <<EOF
OPTIONS

  --help        -h    help
  --version     -v    version
  --nocolor     -n    no color
  --fields      -f    specify options you want displayed
                      Possible 'fields' are:
                        loc     = "Location"
                        time    = "Time"
                        tzone   = "Time Zone"
                        sun     = "Sun Rise / Set"
                        temp    = "Temperature"
                        hmid    = "Humidity"
                        text    = "Current Conditions"
                        bar     = "Barometer
                        dewp    = "Dew Point"
                        vis     = "Visibility"
                        wind    = "Wind"
                        uv      = "UV"
                      See EXAMPLES
EOF

EXAMPLES_TEXT = <<EOF
EXAMPLES

  shell $> rweather -m advanced -- -f temp,humid
  shell $> rweather -m advanced -- -f text,wind
  shell $> rweather -m advanced -- -f list
EOF

module WeatherOut

  def modhelp_advanced
    puts OPTIONS_TEXT
    puts EXAMPLES_TEXT
  end

  def mod_advanced(xml)

    caoff = Colors::COLORS["off"]
    catime = Colors::COLORS["bold-white"]
    catzone = Colors::COLORS["bold-white"]
    caloc = Colors::COLORS["bold-green"]
    catemp = Colors::COLORS["bold-yellow"]
    calnlt = Colors::COLORS["green"]
    casrss = Colors::COLORS["cyan"]
    catzone = Colors::COLORS["white"]
    cahmid = Colors::COLORS["bold-red"]
    catext = Colors::COLORS["magenta"]
    cabar = Colors::COLORS["white"]
    cawind = Colors::COLORS["white"]
    cavis = Colors::COLORS["white"]
    cauv = Colors::COLORS["white"]
    cadewp = Colors::COLORS["white"]

    adv = {
      "time" => xml.elements["weather/cc/lsup"].text,
      "tzone" => xml.elements["weather/loc/zone"].text,
      "loc" => xml.elements["weather/loc/dnam"].text,
      "lat" => xml.elements["weather/loc/lat"].text,
      "lon" => xml.elements["weather/loc/lon"].text,
      "sunr" => xml.elements["weather/loc/sunr"].text,
      "suns" => xml.elements["weather/loc/suns"].text,
      "temp" => xml.elements["weather/cc/tmp"].text,
      "hmid" => xml.elements["weather/cc/hmid"].text,
      "ftext" => xml.elements["weather/cc/t"].text,
      "barr" => xml.elements["weather/cc/bar/r"].text,
      "bard" => xml.elements["weather/cc/bar/d"].text,
      "vis"  => xml.elements["weather/cc/vis"].text,
      "wind_s" => xml.elements["weather/cc/wind/s"].text,
      "wind_d" => xml.elements["weather/cc/wind/t"].text,
      "wind_g" => xml.elements["weather/cc/wind/gust"].text,
      "uv_i" => xml.elements["weather/cc/uv/i"].text,
      "uv_t" => xml.elements["weather/cc/uv/t"].text,
      "dewp" => xml.elements["weather/cc/dewp"].text
    }
   
    out_fields = [
      ["loc", "Location    [ #{caloc}#{adv['loc']}#{caoff} Lt/Ln: #{calnlt}#{adv['lat']}/#{adv['lon']}#{caoff} ]"],
      ["time", "Time        [ #{catime}#{adv['time']}#{caoff} ]"],
      ["tzone", "Time Zone   [ #{catzone}#{adv['tzone']} GMT#{caoff} ]"],
      ["sun", "Sun         [ Rise: #{casrss}#{adv['sunr']}#{caoff} Set: #{casrss}#{adv['suns']}#{caoff} ]"],
      ["temp", "Temp        [ #{catemp}#{adv['temp']}#{UNITS['temp']}#{caoff} ]"],
      ["hmid", "Humidity    [ #{cahmid}#{adv['hmid']}%#{caoff} ]"],
      ["text", "Forecast    [ #{catext}#{adv['ftext']}#{caoff} ]"],
      ["bar", "Barometer   [ #{cabar}#{adv['barr']} #{adv['bard']}#{caoff} ]"],
      ["dewp", "Dew Pt      [ #{cadewp}#{adv['dewp']}#{caoff} ]"],
      ["vis", "Visibility  [ #{cavis}#{adv['vis']}#{caoff} ]"],
      ["wind", "Wind        [ #{cawind}#{adv['wind_s']} #{adv['wind_d']}#{caoff} ]"],
      ["uv", "UV          [ #{cauv}#{adv['uv_i']} #{adv['uv_t']}#{caoff} ]"]
    ]

    def Array::has_key?(key)
      self.each { |k, data| return true if k == key }
      return false
    end


    fields = [ "loc", "time", "tzone", "sun", 
               "temp", "hmid", "text", "bar",
               "dewp", "vis", "wind", "uv" ]


    rcxml = RC["rcxml"]

    if rcxml then
      rcxml.elements.each("rweather/modules/advanced") { |adv|
        adv.elements.each("colors/*") { |color|
          next if color.text == nil
          case color.name
          when "time" then catime = Colors::COLORS["color.text"]
          when "tzone" then catzone = Colors::COLORS["color.text"]
          when "location" then caloc = Colors::COLORS["color.text"]
          when "latitude_longitude" then calnlt = Colors::COLORS["color.text"]
          when "sunrise-set" then casrss = Colors::COLORS["color.text"]
          when "temperature" then catemp = Colors::COLORS["color.text"]
          when "humidity" then cahmid = Colors::COLORS["color.text"]
          when "forecast" then catext = Colors::COLORS["color.text"]
          when "barameter" then cabar = Colors::COLORS["color.text"]
          when "visibility" then cavis = Colors::COLORS["color.text"]
          when "wind" then cawind = Colors::COLORS["color.text"]
          when "uv" then cauv = Colors::COLORS["color.text"]
          when "dewp" then cadewp = Colors::COLORS["color.text"]
          end
        }

        fields = [ ] if adv.elements.each("fields/*").size > 0
        adv.elements.each("fields/*") { |field|
          next if field.text == nil 
          fields << fields.text if out_fields.has_key?(field.text) 
        }
      }
    end


    UNITS["temp"] = "" + xml.elements["weather/head/ut"].text

     
    opts = GetoptLong.new(
      [ "--no-color",   "-n", GetoptLong::NO_ARGUMENT ],
      [ "--version",    "-v", GetoptLong::NO_ARGUMENT ],
      [ "--help",       "-h", GetoptLong::NO_ARGUMENT ],
      [ "--options",    "-o", GetoptLong::REQUIRED_ARGUMENT ],
      [ "--fields",     "-f", GetoptLong::REQUIRED_ARGUMENT ]
    )

    opts.each { |opt, arg|
      case opt
      when "--help"
        modhelp_advanced
        exit 0
      when "--no-color" then caoff, catime, catzone, caloc, catemp, calnlt, casrss, catzone, cahmid, catext, cabar, cawind, cavis, cauv, cadewp = ""
      when "--version" then
        puts MOD_VERSION
        exit 0
      when "--fields" then fields = arg.split(/,/)
      end
    }
    
    fields.each { |f| out_fields.each { |key,frmt| puts frmt if f == key } }
  end
end
